@@ -3,6 +3,8 @@
 #include <linux/rtnetlink.h>
 
 #include "alloc-util.h"
+#include "logarithm.h"
+#include "missing_threads.h"
 #include "networkd-address.h"
 #include "networkd-link.h"
 #include "networkd-manager.h"
@@ -37,6 +39,14 @@ unsigned routes_max(void) {
         return cached;
 }
 
+static bool route_lifetime_is_valid(const Route *route) {
+        assert(route);
+
+        return
+                route->lifetime_usec == USEC_INFINITY ||
+                route->lifetime_usec > now(CLOCK_BOOTTIME);
+}
+
 static Route *link_find_default_gateway(Link *link, int family, Route *gw) {
         Route *route;
 
@@ -121,6 +131,8 @@ bool gateway_is_ready(Link *link, bool onlink, int family, const union in_addr_u
         SET_FOREACH(route, link->routes) {
                 if (!route_exists(route))
                         continue;
+                if (!route_lifetime_is_valid(route))
+                        continue;
                 if (route->family != family)
                         continue;
                 if (!in_addr_is_set(route->family, &route->dst) && route->dst_prefixlen == 0)
@@ -167,6 +179,9 @@ static int link_address_is_reachable_internal(
                 if (!route_exists(route))
                         continue;
 
+                if (!route_lifetime_is_valid(route))
+                        continue;
+
                 if (route->type != RTN_UNICAST)
                         continue;
 
@@ -449,13 +464,12 @@ int config_parse_route_table_names(
                 void *data,
                 void *userdata) {
 
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(userdata);
 
         if (isempty(rvalue)) {
                 m->route_table_names_by_number = hashmap_free(m->route_table_names_by_number);
